package uk.ac.ucl.cs.slangta.editor.actions;



import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;


import uk.ac.ucl.cs.slangta.jmi.impl.slang.SLA_Impl;
import uk.ac.ucl.cs.slangta.jmi.slang.es.InputThroughputScheduledSlidingPenaltyClause;
import uk.ac.ucl.cs.slangta.puppet.AgreementGenerator;
import uk.ac.ucl.cs.slangta.puppet.SLAngPuppetSpecification;
import uk.ac.ucl.cs.slangta.puppet.WorkloadGuaranteeTermsGenerator;



public class GeneratePuppetWizardPage extends WizardPage {
    
	
	private SLA_Impl slangSpecification;
    private String directoryLocation;
    protected Text directoryText;
    protected Text urlText;
    protected Text clientServiceText;
    
    protected Composite container;
    
    private boolean isWorkloadSetted;
    
    public GeneratePuppetWizardPage(SLA_Impl sla) {
        super("generatePuppet");
        slangSpecification = sla;
    }
    
    protected GeneratePuppetWizardPage(String id) {
        super(id);
    }

   
    public void createControl(Composite parent) {

    	// Nothing fancy here: a lot of controls for the form, just used to input
    	// parameters concur, duration, dest. dir for the moment.
    	
    	container = new Composite(parent, SWT.NULL);
                        
        FormLayout layout = new FormLayout();
        layout.marginHeight = 10;
        layout.marginWidth = 10;
        container.setLayout(layout);
        
        GridData layoutData = new GridData();
        layoutData.widthHint = 450;
        layoutData.heightHint = 150;
        container.setLayoutData(layoutData);
        
        Label directoryLabel = new Label(container,SWT.NONE) ;
        Label urlLabel = new Label(container, SWT.NONE);
        Label clientServiceLabel = new Label(container, SWT.NONE);
        
        
        directoryText = new Text(container, SWT.BORDER);
        directoryText.addModifyListener(modifyListener);
        urlText = new Text(container, SWT.BORDER);
        clientServiceText = new Text(container, SWT.BORDER);
        
        
        directoryLabel.setText("Destination Directory:") ;
        directoryLabel.setToolTipText("Choose destination directory");

        urlLabel.setText("URL of Service:");
        urlLabel.setToolTipText("Service Endpoint");

        
        clientServiceLabel.setText("Client Service Name:");
        clientServiceLabel.setToolTipText("The name of the Client's Service");
        
        Button browseButton = new Button(container,SWT.PUSH) ;
        browseButton.setText("Browse");
        browseButton.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                handleDirectoryBrowseButton();
            }
        });
        
        FormData urlLabelLData = new FormData();
        urlLabelLData.top = new FormAttachment(directoryLabel, 30);
        urlLabel.setLayoutData(urlLabelLData);

        FormData clientServiceLabelLData = new FormData();
        clientServiceLabelLData.top = new FormAttachment(urlLabel, 30);
        clientServiceLabel.setLayoutData(clientServiceLabelLData);
        
        FormData directoryTextLData = new FormData();
        directoryTextLData.width = 200;
        directoryTextLData.left = new FormAttachment(directoryLabel, 20);
        directoryText.setLayoutData(directoryTextLData);
        
        FormData urlTextLData = new FormData();
        urlTextLData.width = 200;
        urlTextLData.left = new FormAttachment(directoryLabel, 20);
        urlTextLData.top = new FormAttachment(directoryLabel, 20);
        urlText.setLayoutData(urlTextLData);
        
        
        
        FormData clientServiceTextLData = new FormData();
        clientServiceTextLData.width = 200;
        clientServiceTextLData.left = new FormAttachment(directoryLabel, 20);
        clientServiceTextLData.top = new FormAttachment(urlLabel, 20);
        clientServiceText.setLayoutData(clientServiceTextLData);
        
        
        FormData browseButtonLData = new FormData();
        browseButtonLData.width = 100;
        browseButtonLData.left = new FormAttachment(directoryText, 20);
        browseButton.setLayoutData(browseButtonLData);
        
         
        setControl(container);

    }
    
    public void handleDirectoryBrowseButton() {
        DirectoryDialog dialog = new DirectoryDialog(getShell(), SWT.OPEN);
        directoryLocation = dialog.open();

        if (directoryLocation == null) return;
        directoryText.setText(directoryLocation);
    }
    
        
    protected boolean validatePage() {
    	
    	if ( directoryText.getText().equals("") ) {
    		setMessage("Please select an output directory");
    		return false;
    	}
    	return true;
    }
    
    public String getDirectoryLocation() {
        return directoryLocation;
    }


    
    private ModifyListener modifyListener = new ModifyListener() {
        public void modifyText(ModifyEvent e) {
            setPageComplete(validatePage());
        }
    };
    
    

    
    
    public boolean finish() {
  
    	
    	String destdir = directoryText.getText();
    	String url = urlText.getText();
    	String clientService = clientServiceText.getText();

    	
    	SLAngPuppetSpecification extendedSLAngSpecification = null;
    	
    	
    	
    	
    	
    	
    	// -- A COMMENT TO FRANCO
    	// PLEARE REMOVE THIS STATEMENT
    	isWorkloadSetted = false; 
    	
    	java.util.Collection value = slangSpecification.getAuxiliaryClauses();
		
		if(value != null) {
		
			javax.jmi.reflect.RefObject [] values =
				(javax.jmi.reflect.RefObject [])value.toArray(
					new javax.jmi.reflect.RefObject[value.size()]);
			for(int n = 0; n < values.length; n++) {
				
				String objName = values[n].getClass().getName();
				
				if ( objName == "uk.ac.ucl.cs.slangta.jmi.impl.slang.es.InputThroughputScheduledSlidingPenaltyClause_Impl") {
					isWorkloadSetted = true;
					
				}
			}
		}
    	
    	
    	if (isWorkloadSetted)    	
        	extendedSLAngSpecification = new SLAngPuppetSpecification (slangSpecification,url,clientService);
    	else 
        	extendedSLAngSpecification = new SLAngPuppetSpecification (slangSpecification);

    	AgreementGenerator example = new AgreementGenerator(extendedSLAngSpecification,destdir,"prova.xml");
        example.generate();        

       	
    	return true;

    }

}